import platform
import sys

import boto3

EUI = sys.argv[1]
region = sys.argv[2]
fsb = int(sys.argv[3])
classB = int(sys.argv[4])

client = boto3.client('iotwireless')
iot = boto3.client('iot')

CREDS_PATH = "/user/basic_station/etc/"
LOGS_PATH = "/user/basic_station/"

# 1 : create gateway

name = platform.node()

try:
    lwan = {'GatewayEui': str(EUI), 'RfRegion': str(region)}
    # handle optional sub-bands
    if fsb > 0:
        subbands = []
        for s in range(8):
            if (fsb & (1 << s)):
                subbands.append(s+1)
        lwan['SubBands'] = subbands
    # handle optional class B beaconing
    if classB > 0:
        if region == 'EU868':
            dr = 9
            freq = 869525000
        elif region == 'US915':
            dr = 12
            freq = 923300000
        elif region == 'AU915':
            dr = 10
            freq = 923300000
        elif region == 'AS923' or region == 'AS923-1' or\
                region == 'AS923-2' or region == 'AS923-3' or\
                region == 'AS923-4':
            dr = 9
            freq = 923400000
        elif region == 'KR920':
            dr = 9
            freq = 923400000
        elif region == 'IN865':
            dr = 8
            dr = 866550000
        elif region == 'RU864':
            dr = 9
            freq = 869100000
        else:
            dr = 9
            freq = 869525000
        lwan['Beaconing'] = {'DataRate': dr, 'Frequencies': [freq]}

    gw_desc = "Kerlink gateway created automatically from " + __file__
    print("DESC : " + gw_desc)
    gw_creation = client.create_wireless_gateway(Name=str(name),
                                                 Description=gw_desc,
                                                 LoRaWAN=lwan)
except boto3.exceptions.botocore.exceptions.ClientError as ex:
    print(type(ex).__name__)
    E = ex
    if ex.response['Error']['Code'] == 'ConflictException' and \
       "resource already exists" in ex.response['Error']['Message']:
        print("Gateway " + EUI + " already declared")
        sys.exit(-1)


with open(LOGS_PATH+"gw_creation.log", "w") as f:
    f.write(str(gw_creation)+"\r\n")

GW_ID = gw_creation['Id']
print("Gateway created with ID " + str(GW_ID))

# 2 : create keys and cert

key_and_cert = iot.create_keys_and_certificate(setAsActive=True)

with open(LOGS_PATH+"key_and_cert.log", "w") as f:
    f.write(str(key_and_cert)+"\r\n")

CERT_ID = key_and_cert['certificateId']

cups_key = key_and_cert['keyPair']['PrivateKey']
with open(CREDS_PATH+"cups.key", "w") as f:
    f.write(str(cups_key))
print("cups.key file generated")

cups_crt = key_and_cert['certificatePem']
with open(CREDS_PATH+"cups.crt", "w") as f:
    f.write(str(cups_crt))
print("cups.crt file generated")


# 3 : associate gateway with certificate

assoc_key_cert = client.associate_wireless_gateway_with_certificate(
    Id=str(GW_ID), IotCertificateId=str(CERT_ID))

with open(LOGS_PATH+"assoc_key_cert.log", "w") as f:
    f.write(str(assoc_key_cert)+"\r\n")
print("gateway and certificate associated")


# 4 : retrieve cups server details

get_serv_endpoint = client.get_service_endpoint()
if 'CUPS' != get_serv_endpoint['ServiceType']:
    print("NOT CUPS SERVICE TYPE !")
    sys.exit(-1)

cups_uri = get_serv_endpoint['ServiceEndpoint']
with open(CREDS_PATH+"cups.uri", "w") as f:
    f.write(str(cups_uri))
print("cups.uri file generated")

cups_trust = get_serv_endpoint['ServerTrust']
with open(CREDS_PATH+"cups.trust", "w") as f:
    f.write(str(cups_trust))
print("cups.trust file generated")
